#python

#---Import---
import lx #@UnresolvedImport
import sys
import math
from fs_modoModules.runtime import RunTime
from fs_modoModules.items import Items
from fs_modoModules.channels import Channels
from fs_modoModules.dialogs import Dialogs 
from fs_modoModules.scene import Scene
from fs_modoModules.utilities import Utilities


#---Info---
lx.out("--------------------------------------------------")
lx.out("Author: Fredrik Stenson // fredrik@stenson.tv")
lx.out("www.modo.stenson.tv")
lx.out("Script: fs_AfterFxIO.py")
lx.out("Version: 1.1.201210")
lx.out("")

#---Create Class Instances--- 
runtime = RunTime()
item = Items()
chan = Channels()
dialog = Dialogs()
scene = Scene()
util = Utilities()

args = lx.args()

license = ' License: Commercial \n\n Name: Free for 501'
lx.out(license)

try:

#---License---    
    if 'license' in args:
        
        dialog.info_dialog('License', license)

#---Documentation---
    if 'documentation' in args:
        lx.eval('openURL "http://www.modo.stenson.tv/scripts/commercial/afterfxio/documentation/"')

#---About---    
    if 'about' in args:
        pass
    
#---Import Engine---
    if 'import' in args:
        
#---Check for selected 3d item---
        item_sel = item.q_selected_item('all')
        lx.out(item_sel)
        item_3d = item.check_3D_item(item_sel)
        item_ae_id = ''
        
        for i in range(0, len(item_3d)):
            item_ae_id = item_3d[i]
            item_ae_name = lx.eval('query sceneservice item.name ? %s' %item_3d[i])
            item_ae_type = lx.eval('query sceneservice item.type ? %s' %item_3d[i])
            
        if item_ae_id == '':
            dialog.info_dialog('Select Item', 'You need to select a 3D item')
            sys.exit('No 3D item was selected')
            
        lx.out('Item to import to: %s' %item_ae_name)
        lx.out('Item ID: %s' %item_ae_id)
        lx.out('Item type: %s' %item_ae_type)
        
#---Variables Setup---
        frame_w = float(lx.eval('render.res 0 ?'))
        frame_h = float(lx.eval('render.res 1 ?'))
        rotation_direction = [1, -1, -1]
        scene_divider = lx.eval('user.value scene_divider ?')
        film_w_ae = lx.eval('user.value film_w_ae ?')
        angle_of_view = math.degrees(lx.eval('user.value angle_view_ae ?'))
        focal_check = 0
        
        #match AE rotation order...
        if item_ae_type == 'txtrLocator':
            lx.eval('item.channel txtrLocator(rotation)$order zyx')
        else:
            lx.eval('transform.channel order zyx')
        
        current_time = lx.eval('select.time ?')
        
#---Open Import File---
        file_path = dialog.open_file_dialog('Import After Effects Keyframe Data', 'text')
        file = open(file_path, mode='r')
        
        #read in the file and split it up in an array...
        ae_data = file.read()
        ae_data = ae_data.split()
        
#---Process Imported File---
        for i in range(len(ae_data)):

#---Position Import---
            if 'Transform' ==  ae_data[i] and 'Position' ==  ae_data[i+1]:
                #variable flush...
                frame_or_value = 0
                skip_labels = 9
                
                lx.out('---')
                lx.out(ae_data[i] + ' ' + ae_data[i+1])

                #make sure that it's a numeric character...
                while '1' == ae_data[i+skip_labels][0] or '2' == ae_data[i+skip_labels][0] or '3' == ae_data[i+skip_labels][0] or '4' == ae_data[i+skip_labels][0] or '5' == ae_data[i+skip_labels][0] or '6' == ae_data[i+skip_labels][0] or  '7' == ae_data[i+skip_labels][0] or '8' == ae_data[i+skip_labels][0] or '9' == ae_data[i+skip_labels][0] or '0' == ae_data[i+skip_labels][0] or '-' == ae_data[i+skip_labels][0]:
                    
                    if frame_or_value == 0:
                        lx.out('Frame: ' + ae_data[i+skip_labels])
                        lx.eval("select.time [%s f] [0] [0]" %ae_data[i+skip_labels])
                        
                    if frame_or_value == 1:
                        value = float(ae_data[i+skip_labels]) / scene_divider
                        chan.key_default_xfrm(item_ae_id, 'pos', 'X', value)
                        lx.out('Value posX: %s'  %value)
                        
                    if frame_or_value == 2:
                        value = -float(ae_data[i+skip_labels]) / scene_divider
                        chan.key_default_xfrm(item_ae_id, 'pos', 'Y', value)
                        lx.out('Value posY: %s'  %value)
                        
                    if frame_or_value == 3:
                        value = -float(ae_data[i+skip_labels]) / scene_divider                     
                        chan.key_default_xfrm(item_ae_id, 'pos', 'Z', value)
                        lx.out('Value posZ: %s'  %value)
                                               
                    #extract frames from values counter...    
                    if frame_or_value == 3:
                        frame_or_value = 0
                    else: 
                        frame_or_value = frame_or_value + 1
                    
                    skip_labels = skip_labels + 1
                lx.out('Transformation imported')
                
#---Rotation Import---
            if 'Transform' ==  ae_data[i] and 'Rotation' ==  ae_data[i+1]:
                #variable flush...
                frame_or_value = 0
                skip_labels = 4
                
                lx.out('---')
                lx.out(ae_data[i] + ' ' + ae_data[i+1])

                #make sure that it's a numeric character...
                while '1' == ae_data[i+skip_labels][0] or '2' == ae_data[i+skip_labels][0] or '3' == ae_data[i+skip_labels][0] or '4' == ae_data[i+skip_labels][0] or '5' == ae_data[i+skip_labels][0] or '6' == ae_data[i+skip_labels][0] or  '7' == ae_data[i+skip_labels][0] or '8' == ae_data[i+skip_labels][0] or '9' == ae_data[i+skip_labels][0] or '0' == ae_data[i+skip_labels][0] or '-' == ae_data[i+skip_labels][0]:
                    
                    if frame_or_value == 0:
                        lx.out('Frame: ' + ae_data[i+skip_labels])
                        lx.eval("select.time [%s f] [0] [0]" %ae_data[i+skip_labels])
                        
                    if frame_or_value == 1:
                        value = float(ae_data[i+skip_labels]) * rotation_direction[2]
                        chan.key_default_xfrm(item_ae_id, 'rot', 'Z', value)
                        lx.out('Value rotZ: %s'  %value)
                                               
                    #extract frames from values counter...    
                    if frame_or_value == 1:
                        frame_or_value = 0
                    else: 
                        frame_or_value = frame_or_value + 1
                    
                    skip_labels = skip_labels + 1
                lx.out('Rotation Z imported')
                
                
            if 'Transform' ==  ae_data[i] and 'X' ==  ae_data[i+1] and 'Rotation' ==  ae_data[i+2]:
                #variable flush...
                frame_or_value = 0
                skip_labels = 5
                
                lx.out('---')
                lx.out(ae_data[i] + ' ' + ae_data[i+1] + ' ' + ae_data[i+2])

                #make sure that it's a numeric character...
                while '1' == ae_data[i+skip_labels][0] or '2' == ae_data[i+skip_labels][0] or '3' == ae_data[i+skip_labels][0] or '4' == ae_data[i+skip_labels][0] or '5' == ae_data[i+skip_labels][0] or '6' == ae_data[i+skip_labels][0] or  '7' == ae_data[i+skip_labels][0] or '8' == ae_data[i+skip_labels][0] or '9' == ae_data[i+skip_labels][0] or '0' == ae_data[i+skip_labels][0] or '-' == ae_data[i+skip_labels][0]:
                    
                    if frame_or_value == 0:
                        lx.out('Frame: ' + ae_data[i+skip_labels])
                        lx.eval("select.time [%s f] [0] [0]" %ae_data[i+skip_labels])
                        
                    if frame_or_value == 1:
                        value = float(ae_data[i+skip_labels]) * rotation_direction[0]
                        chan.key_default_xfrm(item_ae_id, 'rot', 'X', value)
                        lx.out('Value rotX: %s'  %value)
                        
                    #extract frames from values counter...    
                    if frame_or_value == 1:
                        frame_or_value = 0
                    else: 
                        frame_or_value = frame_or_value + 1
                    
                    skip_labels = skip_labels + 1
                lx.out('Rotation X imported')
                
                
            if 'Transform' ==  ae_data[i] and 'Y' ==  ae_data[i+1] and 'Rotation' ==  ae_data[i+2]:
                #variable flush...
                frame_or_value = 0
                skip_labels = 5
                
                lx.out('---')
                lx.out(ae_data[i] + ' ' + ae_data[i+1] + ' ' + ae_data[i+2])

                #make sure that it's a numeric character...
                while '1' == ae_data[i+skip_labels][0] or '2' == ae_data[i+skip_labels][0] or '3' == ae_data[i+skip_labels][0] or '4' == ae_data[i+skip_labels][0] or '5' == ae_data[i+skip_labels][0] or '6' == ae_data[i+skip_labels][0] or  '7' == ae_data[i+skip_labels][0] or '8' == ae_data[i+skip_labels][0] or '9' == ae_data[i+skip_labels][0] or '0' == ae_data[i+skip_labels][0] or '-' == ae_data[i+skip_labels][0]:
                    
                    if frame_or_value == 0:
                        lx.out('Frame: ' + ae_data[i+skip_labels])
                        lx.eval("select.time [%s f] [0] [0]" %ae_data[i+skip_labels])
                        
                    if frame_or_value == 1:
                        value = float(ae_data[i+skip_labels]) * rotation_direction[1]
                        chan.key_default_xfrm(item_ae_id, 'rot', 'Y', value)
                        lx.out('Value rotY: %s'  %value) 
                                               
                    #extract frames from values counter...    
                    if frame_or_value == 1:
                        frame_or_value = 0
                    else: 
                        frame_or_value = frame_or_value + 1
                    
                    skip_labels = skip_labels + 1
                lx.out('Rotation Y imported')
                
#---Import Camera---
            if item_ae_type == 'camera':
                
                if 'Source' ==  ae_data[i] and 'Pixel' ==  ae_data[i+1] and 'Aspect' ==  ae_data[i+2] and 'Ratio' ==  ae_data[i+3]:
                    #variable flush...
                    skip_labels = 4
                    
                    lx.out('---')
                    lx.out(ae_data[i] + ' ' + ae_data[i+1] + ' ' + ae_data[i+2] + ' ' + ae_data[i+3])
                    lx.out('Value: %s' %ae_data[i+skip_labels])
                    if ae_data[i+skip_labels] != '1':
                        lx.out('Pixel Aspect Ratio != 1')
                        yes_no = dialog.yes_no_dialog('Sliding Problem', 'The Pixel Aspect Ratio is not set to 1, \nyou might see sliding problems. \nWish to continue?')
                        if yes_no == None:
                            sys.exit('User Ended - Pixel Aspect Ratio != 1')
                
                
                if 'Units' ==  ae_data[i] and 'Per' ==  ae_data[i+1] and 'Second' ==  ae_data[i+2]:
                    #variable flush...
                    skip_labels = 3
                    
                    lx.out('---')
                    lx.out(ae_data[i] + ' ' + ae_data[i+1] + ' ' + ae_data[i+2])
                   
                    lx.out('Frame Rate: ' + ae_data[i+skip_labels])
                    lx.eval('time.fpsCustom %s' %ae_data[i+skip_labels])
                    
                if 'Source' ==  ae_data[i] and 'Width' ==  ae_data[i+1]:
                    #variable flush...
                    skip_labels = 2
                    
                    lx.out('---')
                    lx.out(ae_data[i] + ' ' + ae_data[i+1])
                   
                    lx.out('Frame Width: ' + ae_data[i+skip_labels])
                    lx.eval('render.res 0 %s' %ae_data[i+skip_labels])

                    
                if 'Source' ==  ae_data[i] and 'Height' ==  ae_data[i+1]:
                    #variable flush...
                    skip_labels = 2
                    
                    lx.out('---')
                    lx.out(ae_data[i] + ' ' + ae_data[i+1])
                   
                    lx.out('Frame Height: ' + ae_data[i+skip_labels])
                    lx.eval('render.res 1 %s' %ae_data[i+skip_labels])
                    
                if 'Camera' ==  ae_data[i] and 'Options' ==  ae_data[i+1] and 'Zoom' ==  ae_data[i+2]:
                    #variable flush...
                    frame_or_value = 0
                    skip_labels = 5
                    
                    
                    lx.out('---')
                    lx.out(ae_data[i] + ' ' + ae_data[i+1] + ' ' + ae_data[i+2])
    
                    #make sure that it's a numeric character...
                    while '1' == ae_data[i+skip_labels][0] or '2' == ae_data[i+skip_labels][0] or '3' == ae_data[i+skip_labels][0] or '4' == ae_data[i+skip_labels][0] or '5' == ae_data[i+skip_labels][0] or '6' == ae_data[i+skip_labels][0] or  '7' == ae_data[i+skip_labels][0] or '8' == ae_data[i+skip_labels][0] or '9' == ae_data[i+skip_labels][0] or '0' == ae_data[i+skip_labels][0] or '-' == ae_data[i+skip_labels][0]:
                        
                        if frame_or_value == 0:
                            lx.out('Frame: ' + ae_data[i+skip_labels])
                            lx.eval("select.time [%s f] [0] [0]" %ae_data[i+skip_labels])
                            
                        if frame_or_value == 1:
                            zoom = float(ae_data[i+skip_labels])
                            lx.out('Value Zoom: %s'  %zoom)
                            focal_length = film_w_ae*(zoom/frame_w)
                            chan_id = ('%s:focalLen' %item_ae_id)
                            chan.key_channel_id(chan_id, focal_length)
                            lx.out('Focal Length: %s'  %focal_length)
                            
                        focal_check = focal_check + 1                    
                        #extract frames from values counter...    
                        if frame_or_value == 1:
                            frame_or_value = 0
                        else: 
                            frame_or_value = frame_or_value + 1
                        
                        skip_labels = skip_labels + 1
                    lx.out('Focal Length imported')

               
#---Close Import File---
        file.close()

#---Setup Camera---
        if item_ae_type == 'camera':
            #set the film height for the camera...
            film_h = film_w_ae/(frame_w/frame_h)
            lx.out(film_h)
            lx.eval('item.channel apertureX %s' %film_w_ae)
            lx.eval('item.channel apertureY %s' %film_h)
            
            #if zoom not is animated...
            if focal_check < 2:
                #set the focal length thru the angle of view...
                lx.eval('camera.hfov  %s' %angle_of_view)

#---Reset Time---
        lx.eval('select.time %s' %current_time)
           
#---Export Engine---
    if 'export' in args:
        
#---Check for selected 3d item---
        item_sel = item.q_selected_item('all')
        item_3d = item.check_3D_item(item_sel)
        item_ae_id = ''
        
        for i in range(0, len(item_3d)):
            item_ae_id = item_3d[i]
            item_ae_name = lx.eval('query sceneservice item.name ? %s' %item_3d[i])
            item_ae_type = lx.eval('query sceneservice item.type ? %s' %item_3d[i])
            
        if item_ae_id == '':
            dialog.info_dialog('Select Item', 'You need to select a 3D item')
            sys.exit('No 3D item was selected')
            
        lx.out('Item to export: %s' %item_ae_name)
        lx.out('Item ID: %s' %item_ae_id)
        lx.out('Item type: %s' %item_ae_type)
        
        if item_ae_type == 'camera':
            film_w = lx.eval('item.channel apertureX ?')
            film_h = lx.eval('item.channel apertureY ?')
            film_ratio = film_w / film_h
            
            lx.out('Film Back: %s x %s' %(film_w, film_h))
        
#---Create export file---
        file_path = dialog.save_file_dialog('Export After Effects Keyframe Data', 'text')
        lx.out('File to export: %s' %file_path[0])
        file = open(file_path[0], mode='w+')
        
#---Variables Setup---
        epsilon = 0.00000001
        frames_write = []
        pos_x = []
        pos_y = []
        pos_z = []
        rot_x = []
        rot_y = []
        rot_z = []
        zoom = []
        rot_xyz = []
        rotation_direction = [1, -1, -1]
        rot_tolerance = math.degrees(lx.eval('user.value rot_tolerance ?'))
        scene_multiplier = lx.eval('user.value scene_multiplier ?')
        world = lx.eval('user.value world ?')
        frame_adder = lx.eval('user.value frame_adder ?')
        
        current_time = lx.eval('select.time ?')
        frame_range = scene.q_frame_range()
        fps = frame_range[0]
        lx.out('The framrate is: %s' %fps)
        
        start_frame = frame_range[1] + frame_adder
        end_frame = frame_range[2]
        total_frames = frame_range[3] - frame_adder
        
        lx.out('Start Frame: %s' %start_frame)
        lx.out('End Frame: %s' %end_frame)
        lx.out('Total Frames: %s' %total_frames)
        
        frame_w = float(lx.eval('render.res 0 ?'))
        frame_h = float(lx.eval('render.res 1 ?'))
        frame_ratio = frame_w / frame_h
        
        lx.out('Render Res: %s x %s' %(frame_w, frame_h))
        
#---Ratio check, avoid sliding---   
        if item_ae_type == 'camera':
            if math.fabs(frame_ratio - film_ratio) > epsilon:
                lx.out('frame_ratio != film_ratio')
                yes_no = dialog.yes_no_dialog('Sliding Problem', 'The ratio of the frame do NOT match the film back ratio, \nyou might see sliding problems in After Effects. \nWish to continue?')
                if yes_no == None:
                    sys.exit('User Ended - frame_ratio != film_ratio')
    
#---SetUp Monitor---
        monitor = lx.Monitor()
        monitor.init(total_frames)
        
#---Loop through the scene---
        for i in range(start_frame, end_frame +1):
            
            monitor.step(1)
            
            lx.eval("select.time [%s f] [0] [0]" %i)
            frames_write.append(i)
            
            if item_ae_type == 'camera':
                focal_length = lx.eval('item.channel camera$focalLen ? %s' %item_ae_id)
                zoom.append(frame_w*(focal_length/film_w))
            
            if world == 'world':
                pos_xyz = lx.eval('query sceneservice item.worldPos ? %s' %item_ae_id)
                pos_x.append(scene_multiplier * pos_xyz[0])
                pos_y.append(-scene_multiplier * pos_xyz[1])
                pos_z.append(-scene_multiplier * pos_xyz[2])
               
                cur_rotXYZ = util.change_rot_order('world', item_ae_id, 'zyx', rotation_direction)
                
                if i == start_frame:
                    lx.out('start rot changes')
                    lx.out(rot_tolerance)
                    prev_rotXYZ = cur_rotXYZ
                
                rot_xyz = util.unspike_degrees(prev_rotXYZ, cur_rotXYZ, rot_tolerance)
                
                prev_rotXYZ = rot_xyz
                
                rot_x.append(rot_xyz[0])
                rot_y.append(rot_xyz[1])
                rot_z.append(rot_xyz[2])
    
            if world == 'local':
                pos_x.append((scene_multiplier * lx.eval('item.channel pos.X ?')))
                pos_y.append((-scene_multiplier * lx.eval('item.channel pos.Y ?')))
                pos_z.append((-scene_multiplier * lx.eval('item.channel pos.Z ?')))
                
                #if the items rotation order matches AE:s then skip the convertion... 
                if lx.eval('query sceneservice item.rotOrder ? %s' %item_ae_id) == 5:
                    if i == start_frame:
                        lx.out('Skip Rotation Convertion')
                    cur_xyz = lx.eval('query sceneservice item.rot ? camera004')
                    
                    rot_xyz.append(math.degrees(cur_xyz[0])*rotation_direction[0])
                    rot_xyz.append(math.degrees(cur_xyz[1])*rotation_direction[1])
                    rot_xyz.append(math.degrees(cur_xyz[2])*rotation_direction[2])
                    
                else:
                    
                    cur_rotXYZ = util.change_rot_order('local', item_ae_id, 'zyx', rotation_direction)
                    
                    if i == start_frame:
                        lx.out('start rot changes')
                        lx.out(rot_tolerance)
                        prev_rotXYZ = cur_rotXYZ
                    
                    rot_xyz = util.unspike_degrees(prev_rotXYZ, cur_rotXYZ, rot_tolerance)
                    
                    prev_rotXYZ = rot_xyz
                
                rot_x.append(rot_xyz[0])
                rot_y.append(rot_xyz[1])
                rot_z.append(rot_xyz[2])
                  
#---Write file---
        file.write('Adobe After Effects 8.0 Keyframe Data\n\n')
        file.write('\tUnits Per Second\t%s\n' %fps)
        file.write('\tSource Width\t%s\n' %frame_w)
        file.write('\tSource Height\t%s\n' %frame_h)
        file.write('\tSource Pixel Aspect Ratio\t1\n')
        file.write('\tComp Pixel Aspect Ratio\t1\n')
        file.write('\n')
        
        if item_ae_type == 'camera':
            file.write('Camera Options\tZoom\n')
            file.write('\tFrame\tpixels\n')
            for i in range(len(frames_write)):
                file.write('\t%s\t%s\n' %(frames_write[i], zoom[i]))
        
        file.write('\n')
        
        file.write('Transform\tPosition\n')
        file.write('\tFrame\tX pixels\tY pixels\tZ pixels\n')
        for i in range(len(frames_write)):
            file.write('\t%s\t%s\t%s\t%s\n' %(frames_write[i], pos_x[i], pos_y[i], pos_z[i]))
            
        file.write('\n')
        
        file.write('Transform\tRotation\n')
        file.write('\tFrame\tdegrees\n')
        for i in range(len(frames_write)):
            file.write('\t%s\t%s\n' %(frames_write[i], rot_z[i]))
            
        file.write('\n')
        
        file.write('Transform\tX Rotation\n')
        file.write('\tFrame\tdegrees\n')
        for i in range(len(frames_write)):
            file.write('\t%s\t%s\n' %(frames_write[i], rot_x[i]))
        
        file.write('\n')
        
        file.write('Transform\tY Rotation\n')
        file.write('\tFrame\tdegrees\n')
        for i in range(len(frames_write)):
            file.write('\t%s\t%s\n' %(frames_write[i], rot_y[i]))
            
#---Close File---
        file.write('\n')
        file.write('End of Keyframe Data')
        file.close()
        lx.out('File Exported')
        
#---Reset Time---
        lx.eval('select.time %s' %current_time)    
    
except:
    runtime.error_script()
    
lx.out("--------------------------------------------------")
